unit ASEMesh;

interface

uses
    GL,
    SysUtils,
    BaseTypes,
    BaseMath,
    BaseMesh,
    GLTexture,
    ASE,
    Dialogs,
    Classes,
    BoundingVolumes,
    Camera;

type
    TMeshASE=class(TMesh)
                  private
                  fASEFile:TASEFile;
                  fVNormals:boolean;
                  fTextures:array of TTexture;
                  fNumDrawn:integer;
                  protected
                  procedure RenderMesh(m:TASEMesh);
                  //Constraint for now, you can load only one texture per mesh.
                  //If you want multiple ones, then patch them together
                  //and map it properly :)
                  procedure LoadTextures;
                  function GetTexture(i: integer): TTexture;
                  procedure SetTexture(i: integer; const Value: TTexture);
                  procedure GetOriginAndRadius(m: TASEMesh;var Origin:TVector;var Radius:TFloat);
                  public
                  constructor Create;override;
                  destructor Destroy;override;
                  function LoadFromFile(aFilename:string):boolean;override;
                  procedure Update(TickCount:cardinal);override;
                  procedure Render(aCamera:TCamera);override;
                  procedure CalculateBoundingSphere;override;
                  property VNormals:boolean
                           read fVNormals
                           write fVNormals;
                  property Textures[i:integer]:TTexture
                           read GetTexture
                           write SetTexture;
                  property NumDrawn:integer
                           read fNumDrawn;
    end;

implementation

{ TASEMesh }

constructor TMeshASE.Create;
begin
     fASEFile:=TASEFile.Create;
     Transform:=GetIdentity;
     TextureDir:='';
end;

destructor TMeshASE.Destroy;
begin
     fASEFile.Free;
end;

procedure TMeshASE.RenderMesh(m: TASEMesh);
var
  f: Integer;
begin
  //TODO:Add support for additional stuff like blending, etc.
  fTextures[m.FaceList.Faces[0].MtlID].Enable;
  fTextures[m.FaceList.Faces[0].MtlID].Opaque;

  //glMultMatrixf(@Transform[0,0]);
  with m do
  begin
    glBegin(GL_TRIANGLES);
      for f := 0 to FaceList.NumFaces - 1 do
      begin
        if not fvnormals then
        begin
          if (Normals <> nil) and (Normals.NumFaceNormals > 0) then
            glNormal3f(Normals.FaceNormals[f].X,
                       Normals.FaceNormals[f].Y,
                       Normals.FaceNormals[f].Z);
        end;

        if vnormals then
          if (Normals <> nil) and (Normals.NumVertexNormals > 0) then
            glNormal3f(Normals.VertexNormals[FaceList.Faces[f].A].X,
                       Normals.VertexNormals[FaceList.Faces[f].A].Y,
                       Normals.VertexNormals[FaceList.Faces[f].A].Z);

        glTexCoord2f(TVertexList.TVertices[TFaceList.TFaces[f].A].U,TVertexList.TVertices[TFaceList.TFaces[f].A].V);

        glVertex3f(VertexList.Vertices[FaceList.Faces[f].A].X,
                   VertexList.Vertices[FaceList.Faces[f].A].Y,
                   VertexList.Vertices[FaceList.Faces[f].A].Z);

        if vnormals then
          if (Normals <> nil) and (Normals.NumVertexNormals > 0) then
            glNormal3f(Normals.VertexNormals[FaceList.Faces[f].B].X,
                       Normals.VertexNormals[FaceList.Faces[f].B].Y,
                       Normals.VertexNormals[FaceList.Faces[f].B].Z);

        glTexCoord2f(TVertexList.TVertices[TFaceList.TFaces[f].B].U,TVertexList.TVertices[TFaceList.TFaces[f].B].V);

        glVertex3f(VertexList.Vertices[FaceList.Faces[f].B].X,
                   VertexList.Vertices[FaceList.Faces[f].B].Y,
                   VertexList.Vertices[FaceList.Faces[f].B].Z);

        if vnormals then
          if (Normals <> nil) and (Normals.NumVertexNormals > 0) then
            glNormal3f(Normals.VertexNormals[FaceList.Faces[f].C].X,
                       Normals.VertexNormals[FaceList.Faces[f].C].Y,
                       Normals.VertexNormals[FaceList.Faces[f].C].Z);

        glTexCoord2f(TVertexList.TVertices[TFaceList.TFaces[f].C].U,TVertexList.TVertices[TFaceList.TFaces[f].C].V);

        glVertex3f(VertexList.Vertices[FaceList.Faces[f].C].X,
                   VertexList.Vertices[FaceList.Faces[f].C].Y,
                   VertexList.Vertices[FaceList.Faces[f].C].Z);
      end;
    glEnd;

  end;
end;


function TMeshASE.LoadFromFile(aFilename: string): boolean;
begin
     result:=false;
     try
     fASEFile.LoadFromFile(aFilename);
     LoadTextures;
     except
           exit;
     end;
     result:=true;
end;

procedure TMeshASE.Render(aCamera:TCamera);
var
   o,g:integer;
begin
  fNumDrawn:=0;
  glPushMatrix;
  glRotatef(-90,1,0,0);
  for o := 0 to fASEFile.NumGeomObjects - 1 do
  begin
       if not (BoundingSphereCompletelyInFrustum(fASEFile.GeomObjects[o].SphereBoundingVolume,aCamera.Frustum)=PARTIALLY_IN_FRUSTUM) then
       begin
            RenderMesh(fASEFile.GeomObjects[o].Mesh);
            Inc(fNumDrawn);
       end;
  end;

  for g := 0 to fASEFile.NumGroups - 1 do
  begin
    for o := 0 to fASEFile.Groups[g].NumGeomObjects - 1 do
    begin
         if not (BoundingSphereCompletelyInFrustum(fASEFile.Groups[g].GeomObjects[o].SphereBoundingVolume,aCamera.Frustum)=PARTIALLY_IN_FRUSTUM) then
         begin
              RenderMesh(fASEFile.Groups[g].GeomObjects[o].Mesh);
              Inc(fNumDrawn);
         end;
    end;
  end;
  glPopMAtrix;
end;

procedure TMeshASE.Update(TickCount: cardinal);
begin
     //nothing....
end;

function TMeshASE.GetTexture(i: integer): TTexture;
begin
     result:=fTextures[i];
end;

procedure TMeshASE.SetTexture(i: integer; const Value: TTexture);
begin
     fTextures[i]:=value;
end;

procedure TMeshASE.LoadTextures;
var
   i:integer;
begin
     //All meshes *have* to be textured.
     //You want a plain color? use a 2x2 plain bitmap. Sorry!
     for i:=0 to fASEFile.MaterialList.NumMaterials-1 do
     begin
          SetLength(fTextures,i+1);
          fTextures[i]:=TTexture.Create;
          fTextures[i].MIPMapping:=true;
          //Base texture is the diffuse bitmap
          if not fTextures[i].LoadTexture(TextureDir+ExtractFilename(fASEFile.MaterialList.Materials[i].Maps[ASE_DIFFUSE_MAP].Bitmap)) then
             ShowMessage('Could not load texture '+TextureDir+ExtractFilename(fASEFile.MaterialList.Materials[i].Maps[ASE_DIFFUSE_MAP].Bitmap));

          fTextures[i].Name:=ExtractFilename(fASEFile.MaterialList.Materials[i].Maps[ASE_DIFFUSE_MAP].Bitmap);
     end;
end;

procedure TMeshASE.GetOriginAndRadius(m: TASEMesh;var Origin:TVector;var Radius:TFloat);
var
  f: Integer;
  tempVector:TVector;
  minX,minY,minZ,
  maxX,maxY,maxZ:TFloat;
begin
     with m do
     begin
          //Gimme me a break. There has to be at least one face in a geometric object!!
          minX:=VertexList.Vertices[FaceList.Faces[0].A].X;
          minY:=VertexList.Vertices[FaceList.Faces[0].A].Y;
          minZ:=VertexList.Vertices[FaceList.Faces[0].A].Z;

          maxX:=VertexList.Vertices[FaceList.Faces[0].A].X;
          maxY:=VertexList.Vertices[FaceList.Faces[0].A].Y;
          maxZ:=VertexList.Vertices[FaceList.Faces[0].A].Z;

          for f := 0 to FaceList.NumFaces - 1 do
          begin
               tempVector:=VectorAdd(tempVector,MakeVector(VertexList.Vertices[FaceList.Faces[f].A].X,
                                                           VertexList.Vertices[FaceList.Faces[f].A].Y,
                                                           VertexList.Vertices[FaceList.Faces[f].A].Z));
               minX:=Min(VertexList.Vertices[FaceList.Faces[f].A].X,minX);
               minY:=Min(VertexList.Vertices[FaceList.Faces[f].A].Y,minY);
               minZ:=Min(VertexList.Vertices[FaceList.Faces[f].A].Z,minZ);
               maxX:=Max(VertexList.Vertices[FaceList.Faces[f].A].X,maxX);
               maxY:=Max(VertexList.Vertices[FaceList.Faces[f].A].Y,maxY);
               maxZ:=Max(VertexList.Vertices[FaceList.Faces[f].A].Z,maxZ);

               tempVector:=VectorAdd(tempVector,MakeVector(VertexList.Vertices[FaceList.Faces[f].B].X,
                                                           VertexList.Vertices[FaceList.Faces[f].B].Y,
                                                           VertexList.Vertices[FaceList.Faces[f].B].Z));
               minX:=Min(VertexList.Vertices[FaceList.Faces[f].B].X,minX);
               minY:=Min(VertexList.Vertices[FaceList.Faces[f].B].Y,minY);
               minZ:=Min(VertexList.Vertices[FaceList.Faces[f].B].Z,minZ);
               maxX:=Max(VertexList.Vertices[FaceList.Faces[f].B].X,maxX);
               maxY:=Max(VertexList.Vertices[FaceList.Faces[f].B].Y,maxY);
               maxZ:=Max(VertexList.Vertices[FaceList.Faces[f].B].Z,maxZ);

               tempVector:=VectorAdd(tempVector,MakeVector(VertexList.Vertices[FaceList.Faces[f].C].X,
                                                           VertexList.Vertices[FaceList.Faces[f].C].Y,
                                                           VertexList.Vertices[FaceList.Faces[f].C].Z));
               minX:=Min(VertexList.Vertices[FaceList.Faces[f].C].X,minX);
               minY:=Min(VertexList.Vertices[FaceList.Faces[f].C].Y,minY);
               minZ:=Min(VertexList.Vertices[FaceList.Faces[f].C].Z,minZ);
               maxX:=Max(VertexList.Vertices[FaceList.Faces[f].C].X,maxX);
               maxY:=Max(VertexList.Vertices[FaceList.Faces[f].C].Y,maxY);
               maxZ:=Max(VertexList.Vertices[FaceList.Faces[f].C].Z,maxZ);
          end;
     end;
     Origin:=MakeVector((minX+maxX)/2,(minY+maxY)/2,(minZ+minZ)/2);
     Origin:=VectorScale(1/m.VertexList.NumVertices,Origin);
     Radius:=Sqrt(SquaredCartesianDistance(Origin,VectorSubtract(Origin,MakeVector(maxX,maxY,maxZ))));
end;


procedure TMeshASE.CalculateBoundingSphere;
var
   o,g:integer;
begin
  for o := 0 to fASEFile.NumGeomObjects - 1 do
  begin
       fASEFile.GeomObjects[o].SphereBoundingVolume:=TSphereBoundingVolume.Create;
       GetOriginAndRadius(fASEFile.GeomObjects[o].Mesh,
                          PSphereBoundRec(fASEFile.GeomObjects[o].SphereBoundingVolume.BoundsRec)^.Origin,
                          PSphereBoundRec(fASEFile.GeomObjects[o].SphereBoundingVolume.BoundsRec)^.Radius);
  end;

  for g := 0 to fASEFile.NumGroups - 1 do
  begin
    for o := 0 to fASEFile.Groups[g].NumGeomObjects - 1 do
    begin
         fASEFile.Groups[g].GeomObjects[o].SphereBoundingVolume:=TSphereBoundingVolume.Create;
         GetOriginAndRadius(fASEFile.Groups[g].GeomObjects[o].Mesh,
                            PSphereBoundRec(fASEFile.Groups[g].GeomObjects[o].SphereBoundingVolume.BoundsRec)^.Origin,
                            PSphereBoundRec(fASEFile.Groups[g].GeomObjects[o].SphereBoundingVolume.BoundsRec)^.Radius);
    end;
  end;
end;

end.
