unit Base3DObject;

interface

uses
    Windows,
    GL,
    GLUT,
    BaseTypes,
    BaseMath,
    BoundingVolumes,
    Camera;

type
    //Gives basic time keyed behavior
    TBaseObject=class
                     private
                     fFrameInterval,fLastTickCount:cardinal;
                     fEnabled:boolean;
                     public
                     function Update(TickCount:cardinal):boolean;virtual;
                     constructor Create;
                     procedure PreRender;virtual;
                     procedure Render(aCamera:TCamera);virtual;abstract;
                     procedure PostRender;virtual;
                     property FrameInterval:cardinal
                              read fFrameInterval
                              write fFrameInterval;
                     property Enabled:boolean
                              read fEnabled
                              write fEnabled;
    end;
    //Adds transformations and collisions and stuff [RUV]
    T3DObject=class(TBaseObject)
                   private
                   fTranslation:TVector;
                   fR:TVector;
                   fU:TVector;
                   fV:TVector;
                   public
                   constructor Create;
                   procedure RotateAboutR(amt:TFloat);
                   procedure RotateAboutU(amt:TFloat);
                   procedure RotateAboutV(amt:TFloat);
                   function GetRotation:TMatrix;
                   function GetTransform:TMatrix;
                   procedure SetTransform;
                   procedure PreRender;override;

                   property R:TVector
                            read fR;
                   property U:TVector
                            read fU;
                   property V:TVector
                            read fV;
                   property Translation:TVector
                            read fTranslation
                            write fTranslation;
    end;

implementation

constructor TBaseObject.Create;
begin
     fEnabled:=true;
end;

function TBaseObject.Update(TickCount: cardinal): boolean;
begin
     result:=false;
     if not fEnabled then exit;
     if (TickCount-fLastTickCount)>=fFrameInterval then
     begin
          fLastTickCount:=TickCount;
          result:=true;
     end;
end;
procedure TBaseObject.PreRender;
begin
     glPushMatrix;
end;

procedure TBaseObject.PostRender;
begin
     glPopMatrix;
end;

constructor T3DObject.Create;
begin
     inherited;
     fR:=MakeVector(1,0,0);
     fU:=MakeVector(0,1,0);
     fV:=MakeVector(0,0,1);
end;

procedure T3DObject.RotateAboutR(amt:TFloat);
begin
     fR:=VectorRotateX(fR,amt);
     fU:=VectorRotateX(fU,amt);
     fV:=VectorRotateX(fV,amt);
     fR:=VectorNormalize(fR);
     fU:=VectorNormalize(fU);
     fV:=VectorNormalize(fV);
end;
procedure T3DObject.RotateAboutU(amt:TFloat);
begin
     fR:=VectorRotateY(fR,amt);
     fU:=VectorRotateY(fU,amt);
     fV:=VectorRotateY(fV,amt);
     fR:=VectorNormalize(fR);
     fU:=VectorNormalize(fU);
     fV:=VectorNormalize(fV);
end;
procedure T3DObject.RotateAboutV(amt:TFloat);
begin
     fR:=VectorRotateZ(fR,amt);
     fU:=VectorRotateZ(fU,amt);
     fV:=VectorRotateZ(fV,amt);
     fR:=VectorNormalize(fR);
     fU:=VectorNormalize(fU);
     fV:=VectorNormalize(fV);
end;
function T3DObject.GetRotation: TMatrix;
var
   rotMatrix:TMatrix;
begin
     rotMatrix:=GetIdentity;

     rotMatrix[0,0]:=fR[0];
     rotMatrix[1,0]:=fR[1];
     rotMatrix[2,0]:=fR[2];

     rotMatrix[0,1]:=fU[0];
     rotMatrix[1,1]:=fU[1];
     rotMatrix[2,1]:=fU[2];

     rotMatrix[0,2]:=fV[0];
     rotMatrix[1,2]:=fV[1];
     rotMatrix[2,2]:=fV[2];

     result:=rotMatrix;
end;

function T3DObject.GetTransform:TMatrix;
var
   rotMatrix,transMatrix:TMatrix;
begin
     transMatrix:=GetIdentity;
     transMatrix[0,3]:=fTranslation[0];
     transMatrix[1,3]:=fTranslation[1];
     transMatrix[2,3]:=fTranslation[2];

     rotMatrix:=GetIdentity;

     rotMatrix[0,0]:=fR[0];
     rotMatrix[1,0]:=fR[1];
     rotMatrix[2,0]:=fR[2];

     rotMatrix[0,1]:=fU[0];
     rotMatrix[1,1]:=fU[1];
     rotMatrix[2,1]:=fU[2];

     rotMatrix[0,2]:=fV[0];
     rotMatrix[1,2]:=fV[1];
     rotMatrix[2,2]:=fV[2];

     result:=MatrixMultiply(rotMatrix,transMatrix);
end;
procedure T3DObject.SetTransform;
var
   transform:TMatrix;
begin
     transform:=GetTransform;
     transform:=MatrixTranspose(transform);
     glMultMatrixf(@transform[0,0]);
end;

procedure T3DObject.PreRender;
begin
     inherited;
     SetTransform;
end;

end.
