unit BaseMath;

interface

uses
    SysUtils,
    BaseTypes,
    BoundingVolumes;
const
     X = 0;
     Y = 1;
     Z = 2;
     W = 3;
     EPS:TFloat=0.01;
     DEGTORAD=3.1412/180;
     RADTODEG=180/3.1412;
     LAMBDA_INCREMENT:TFloat=0.01;

function GetIdentity:TMatrix;
function CreateRotationMatrixX(const ang:TFloat):TMatrix;register;
function CreateRotationMatrixY(const ang:TFloat):TMatrix;register;
function CreateRotationMatrixZ(const ang:TFloat):TMatrix;register;
function GetZeroVector:TVector;
function MatrixMultiply(const M1,M2:TMatrix):TMatrix;register;
function VectorTransform(const V:TVector;const M:TMatrix):TVector;register;
function VectorScale(s:TFloat;const V:TVector):TVector;register;
function MatrixScale(s:TFloat;const M:TMatrix):TMatrix;register;
function MatrixTranspose(const M:TMatrix):TMatrix;register;
function VectorAdd(V1,V2:TVector):TVector;register;
function VectorSubtract(V1,V2:TVector):TVector;register;
function VectorSquaredLength(V:TVector):TFloat;register;
function ArcTan2(Y,X:extended):extended;register;
function ArcCos(x:single):single;register;
function VectorDotProduct(v1,v2:TVector):TFloat;
function VectorCrossProduct(v1,v2:TVector):TVector;
function VectorRotateX(v:TVector;a:TFloat):TVector;
function VectorRotateY(v:TVector;a:TFloat):TVector;
function VectorRotateZ(v:TVector;a:TFloat):TVector;
function VectorNormalize(v:TVector):TVector;
function GetNormal(a,b,c:TVector):TVector;
function Normalize(aVector:TVector):TVector;
function GetPlaneFromPoints(p1,p2,p3:TVector):TPlane;
function VectorInterpolate(X1,X2:TVector;lambda:single):TVector;
function VectorLength(X:TVector):TFloat;
function SquaredCartesianDistance(V1,V2:TVector):TFloat;
function VectorEqual(v1,v2:T2DVector):boolean;overload;
function VectorEqual(v1,v2:TVector):boolean;overload;
function Interpolate(X1,X2:T2DVector;lambda:TFloat):T2DVector;overload;
function Interpolate(X1,X2:TVector;lambda:TFloat):TVector;overload;
function VectorInFrustum(X:TVector;aFrustum:TFrustum):boolean;
function BoundingSphereInFrustum(aSphere:TSphereBoundingVolume;aFrustum:TFrustum):boolean;
function BoundingSphereCompletelyInFrustum(aSphere:TSphereBoundingVolume;aFrustum:TFrustum):integer;
function Min(a,b:TFloat):TFloat;
function Max(a,b:TFloat):TFloat;
implementation

function GetIdentity:TMatrix;
begin
     result[0,0]:=1.0;result[0,1]:=0.0;result[0,2]:=0.0;result[0,3]:=0.0;
     result[1,0]:=0.0;result[1,1]:=1.0;result[1,2]:=0.0;result[1,3]:=0.0;
     result[2,0]:=0.0;result[2,1]:=0.0;result[2,2]:=1.0;result[2,3]:=0.0;
     result[3,0]:=0.0;result[3,1]:=0.0;result[3,2]:=0.0;result[3,3]:=1.0;
end;
function GetZeroVector:TVector;
begin
     result[0]:=0.0;
     result[1]:=0.0;
     result[2]:=0.0;
end;
function CreateRotationMatrixX(const ang:TFloat): TMatrix;
var
   Sine,Cosine:TFloat;
   rads:TFloat;
begin
     rads:=ang*DEGTORAD;
     Sine:=Sin(rads);
     Cosine:=Cos(rads);

     Result[X, X] := 1;
     Result[X, Y] := 0;
     Result[X, Z] := 0;
     Result[X, W] := 0;

     Result[Y, X] := 0;
     Result[Y, Y] := Cosine;
     Result[Y, Z] := -Sine;
     Result[Y, W] := 0;

     Result[Z, X] := 0;
     Result[Z, Y] := Sine;
     Result[Z, Z] := Cosine;
     Result[Z, W] := 0;

     Result[W, X] := 0;
     Result[W, Y] := 0;
     Result[W, Z] := 0;
     Result[W, W] := 1;
end;

function CreateRotationMatrixY(const ang:TFloat): TMatrix;
var
   Sine,Cosine:TFloat;
   rads:TFloat;
begin
     rads:=ang*DEGTORAD;
     Sine:=Sin(rads);
     Cosine:=Cos(rads);

     Result[X, X] := Cosine;
     Result[X, Y] := 0;
     Result[X, Z] := Sine;
     Result[X, W] := 0;

     Result[Y, X] := 0;
     Result[Y, Y] := 1;
     Result[Y, Z] := 0;
     Result[Y, W] := 0;

     Result[Z, X] := -Sine;
     Result[Z, Y] := 0;
     Result[Z, Z] := Cosine;
     Result[Z, W] := 0;

     Result[W, X] := 0;
     Result[W, Y] := 0;
     Result[W, Z] := 0;
     Result[W, W] := 1;
end;

function CreateRotationMatrixZ(const ang:TFloat): TMatrix;
var
   Sine,Cosine:TFloat;
   rads:TFloat;
begin
     rads:=ang*DEGTORAD;
     Sine:=Sin(rads);
     Cosine:=Cos(rads);

     Result[X, X] := Cosine;
     Result[X, Y] := -Sine;
     Result[X, Z] := 0;
     Result[X, W] := 0;

     Result[Y, X] := Sine;
     Result[Y, Y] := Cosine;
     Result[Y, Z] := 0;
     Result[Y, W] := 0;

     Result[Z, X] := 0;
     Result[Z, Y] := 0;
     Result[Z, Z] := 1;
     Result[Z, W] := 0;

     Result[W, X] := 0;
     Result[W, Y] := 0;
     Result[W, Z] := 0;
     Result[W, W] := 1;
end;

function MatrixMultiply(const M1, M2: TMatrix):TMatrix;
// multiplies two 4x4 matrices
var
   i,j:Integer;
   TM:TMatrix;
begin
     for i := 0 to 3 do
       for j := 0 to 3 do
         TM[i,j]:=M1[i, X] * M2[X, j] +
                  M1[i, Y] * M2[Y, j] +
                  M1[i, Z] * M2[Z, j] +
                  M1[i, W] * M2[W, j];
     result:=TM;
end;
function VectorTransform(const V:TVector;const M:TMatrix):TVector;
begin
     result[X] := V[X] * M[X, X] + V[Y] * M[X, Y] + V[Z] * M[X, Z] + M[X, W];
     result[Y] := V[X] * M[Y, X] + V[Y] * M[Y, Y] + V[Z] * M[Y, Z] + M[Y, W];
     result[Z] := V[X] * M[Z, X] + V[Y] * M[Z, Y] + V[Z] * M[Z, Z] + M[Z, W];
end;
function VectorScale(s:TFloat;const V:TVector):TVector;
var
   TV:TVector;
begin
     TV[X]:=s*V[X];
     TV[Y]:=s*V[Y];
     TV[Z]:=s*V[Z];
     result:=TV;
end;
function VectorAdd(V1,V2:TVector):TVector;
begin
     result[X]:=V1[X]+V2[X];
     result[Y]:=V1[Y]+V2[Y];
     result[Z]:=V1[Z]+V2[Z];
end;
function VectorSubtract(V1,V2:TVector):TVector;register;
begin
     result[X]:=V1[X]-V2[X];
     result[Y]:=V1[Y]-V2[Y];
     result[Z]:=V1[Z]-V2[Z];
end;
function MatrixScale(s:TFloat;const M:TMatrix):TMatrix;register;
var
   i,j:integer;
begin
     for i:=0 to 3 do
         for j:=0 to 3 do
             result[i,j]:=s*M[i,j];
end;
function MatrixTranspose(const M:TMatrix):TMatrix;register;
var
   i,j:integer;
begin
     for i:=0 to 3 do
         for j:=0 to 3 do
             result[i,j]:=M[j,i];
end;
function VectorSquaredLength(V:TVector):TFloat;
begin
     result:=V[0]*V[0]+V[1]*V[1]+V[2]*V[2];
end;
function ArcTan2(Y,X:extended):extended;
asm
   FLD Y
   FLD X
   FPATAN
   FWAIT
end;

function ArcCos(x:single):single;
begin
     Result:=ArcTan2(Sqrt(1-x*x), x);
end;
function VectorDotProduct(v1,v2:TVector):TFloat;
begin
     result:=v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2];     
end;
function VectorCrossProduct(v1,v2:TVector):TVector;
begin
     Result := MakeVector(v1[y] * v2[z] - v2[y] * v1[z],
                        v2[x] * v1[z] - v1[x] * v2[z],
                        v1[x] * v2[y] - v2[x] * v1[y]);
end;

function VectorRotateX(v:TVector;a:TFloat):TVector;
var
   temp: TVector;
   sine,cosine:TFloat;
begin
     a:=a*DEGTORAD;
     sine:=Sin(a);
     cosine:=Cos(a);

     temp[X] := v[x];
     temp[Y]:= (v[Y] * cosine) + (v[Z] * -sine);
     temp[Z] := (v[Y] * sine) + (v[Z] * cosine);
     result := temp;
end;

function VectorRotateY(v: TVector;a:TFloat):TVector;
var
   temp: TVector;
   sine,cosine:TFloat;
begin
     a:=a*DEGTORAD;
     sine:=Sin(a);
     cosine:=Cos(a);

     temp[x] := (v[x] * cosine) + (v[z] * sine);
     temp[y] := v[y];
     temp[z] := (v[x] * -sine) + (v[z] * cosine);
     result := temp;
end;

function VectorRotateZ(v: TVector; a: TFloat):TVector;
var
  temp: TVector;
   sine,cosine:TFloat;
begin
     a:=a*DEGTORAD;
     sine:=Sin(a);
     cosine:=Cos(a);
     temp[x] := (v[x] * cosine) + (v[y] * -sine);
     temp[y] := (v[x] * sin(a)) + (v[y] * cosine);
     temp[z] := v[z];
     result := temp;
end;
function VectorNormalize(v:TVector):TVector;
var
   val:TFloat;
begin
     val:=v[x]*v[x]+
          v[y]*v[y]+
          v[z]*v[z];
     val:=sqrt(val);
     result[x]:=v[x]/val;
     result[y]:=v[y]/val;
     result[z]:=v[z]/val;
end;
function GetNormal(a,b,c:TVector):TVector;
var
   p1,p2:TVector;
begin
     p1[X]:=a[X]-b[X];
     p1[Y]:=a[Y]-b[Y];
     p1[Z]:=a[Z]-b[Z];

     p2[X]:=b[X]-c[X];
     p2[Y]:=b[Y]-c[Y];
     p2[Z]:=b[Z]-c[Z];

     result[X]:=p1[Y]*p2[Z]-p1[Z]*p2[Y];
     result[Y]:=p1[Z]*p2[X]-p1[X]*p2[Z];
     result[Z]:=p1[X]*p2[Y]-p1[Y]*p2[X];
     //result:=Normalize(result);
end;
function Normalize(aVector:TVector):TVector;
var
   d:double;
begin
     d:=Sqrt(Sqr(aVector[X])+Sqr(aVector[Y])+Sqr(aVector[Z]));
     if d=0 then
        raise exception.Create('Zero length vector');
     result[X]:=aVector[X]/d;
     result[Y]:=aVector[Y]/d;
     result[Z]:=aVector[Z]/d;
end;

function GetPlaneFromPoints(p1,p2,p3:TVector):TPlane;
var
   n:TVector;
begin
     n:=GetNormal(p1,p2,p3);
     with result do
     begin
          A:=n[X];
          B:=n[Y];
          C:=n[Z];
          D:=-(A * p1[X] + B * p1[Y] + C * p1[Z]);
     end;
end;
function VectorInterpolate(X1,X2:TVector;lambda:single):TVector;
begin
     result[X]:=X1[X] + (X2[X] - X1[X]) * lambda;
     result[Y]:=X1[Y] + (X2[Y] - X1[Y]) * lambda;
     result[Z]:=X1[Z] + (X2[Z] - X1[Z]) * lambda;
end;
function VectorLength(X:TVector):TFloat;
begin
     result:=Sqrt(X[0]*X[0]+X[1]*X[1]+X[2]*X[2]);
end;

function SquaredCartesianDistance(V1,V2:TVector):TFloat;
begin
     result:=Sqr(V2[X]-V1[X])+
             Sqr(V2[Y]-V1[Y])+
             Sqr(V2[Z]-V1[Z]);
end;
function VectorEqual(v1,v2:T2DVector):boolean;
begin
     result:=((abs(v1[0]-v2[0])<=EPS) and
              (abs(v1[1]-v2[1])<=EPS));
end;
function VectorEqual(v1,v2:TVector):boolean;
begin
     result:=((abs(v1[0]-v2[0])<=EPS) and
              (abs(v1[1]-v2[2])<=EPS) and
              (abs(v1[2]-v2[2])<=EPS));
end;
function Interpolate(X1,X2:T2DVector;lambda:TFloat):T2DVector;
begin
     result[0]:=X1[0] + (X2[0] - X1[0]) * lambda;
     result[1]:=X1[1] + (X2[1] - X1[1]) * lambda;
end;
function Interpolate(X1,X2:TVector;lambda:TFloat):TVector;
begin
     result[0]:=X1[0] + (X2[0] - X1[0]) * lambda;
     result[1]:=X1[1] + (X2[1] - X1[1]) * lambda;
     result[2]:=X1[2] + (X2[2] - X1[2]) * lambda;
end;
function VectorInFrustum(X:TVector;aFrustum:TFrustum):boolean;
var
   p:integer;
begin
     for p:=0 to 5 do
         if (aFrustum[p,0]*X[0]+
            aFrustum[p,1]*X[1]+
            aFrustum[p,2]*X[2]+
            aFrustum[p,3]*X[3])<=0 then
         begin
              result:=false;
              exit;
         end;
     result:=true;
end;
function BoundingSphereInFrustum(aSphere:TSphereBoundingVolume;aFrustum:TFrustum):boolean;
var
   p:integer;
begin
     for p:=0 to 5 do
         if (aFrustum[p,0]*PSphereBoundRec(aSphere.fBoundsRec)^.Origin[0]+
             aFrustum[p,1]*PSphereBoundRec(aSphere.fBoundsRec)^.Origin[1]+
             aFrustum[p,2]*PSphereBoundRec(aSphere.fBoundsRec)^.Origin[2]+
             aFrustum[p,3]*PSphereBoundRec(aSphere.fBoundsRec)^.Origin[3])<=-PSphereBoundRec(aSphere.fBoundsRec)^.Radius then
         begin
              result:=false;
              exit;
         end;
     result:=true;
end;
function BoundingSphereCompletelyInFrustum(aSphere:TSphereBoundingVolume;aFrustum:TFrustum):integer;
var
   p,c:integer;
begin
     c:=0;
     for p:=0 to 5 do
         if (aFrustum[p,0]*PSphereBoundRec(aSphere.fBoundsRec)^.Origin[0]+
             aFrustum[p,1]*PSphereBoundRec(aSphere.fBoundsRec)^.Origin[1]+
             aFrustum[p,2]*PSphereBoundRec(aSphere.fBoundsRec)^.Origin[2]+
             aFrustum[p,3]*PSphereBoundRec(aSphere.fBoundsRec)^.Origin[3])<=-PSphereBoundRec(aSphere.fBoundsRec)^.Radius then
         begin
              result:=NOT_IN_FRUSTUM;
              exit;
         end
         else
             c:=c+1;
     if c=6 then
        result:=TOTALLY_IN_FRUSTUM
     else
         result:=PARTIALLY_IN_FRUSTUM;
end;
function Min(a,b:TFloat):TFloat;
begin
     if a<b then result:=a
     else
         result:=b;
end;
function Max(a,b:TFloat):TFloat;
begin
     if a>b then
        result:=a
     else
         result:=b;
end;

end.

