unit BaseMesh;

interface

uses
    GL,
    SysUtils,
    BaseTypes,
    BaseMath,
    BoundingVolumes,
    GLTexture,
    Camera;

const
     //stuff for my format RAX meshes
     MESH_INDICATOR='#:';
     TEXTURE_INDICATOR='*:';
     INDICES_INDICATOR='[]:';
     VERTICES_INDICATOR='{}:';

type
    TMesh=class
               private
               fName:string;
               fTransform:TMatrix;
               fTextureDir:string;
               fBoundingVolume:TSphereBoundingVolume;
               public
               constructor Create;virtual;
               function LoadFromFile(aFilename:string):boolean;virtual;abstract;
               procedure Render(aCamera:TCamera);virtual;abstract;
               procedure Update(TickCount:cardinal);virtual;abstract;
               procedure CalculateBoundingSphere;virtual;abstract;
               destructor Destroy;override;
               property Name:string
                        read fName
                        write fName;
               property Transform:TMatrix
                        read fTransform
                        write fTransform;
               property TextureDir:string
                        read fTextureDir
                        write fTextureDir;
               property BoundingVolume:TSphereBoundingVolume
                        read fBoundingVolume
                        write FBoundingVolume;
    end;

    TRAXMesh=class(TMesh)
                  private
                  fFaces:TFaceArray;
                  fVertices:TVertexArray;
                  fTexture:TTexture;
                  public
                  constructor Create;override;
                  destructor Destroy;override;
                  function LoadFromFile(aFilename:string):boolean;override;
                  procedure Update(TickCount:cardinal);override;
                  procedure Render(aCamera:TCamera);override;                  
    end;

implementation

constructor TRAXMesh.Create;
begin
     //nothing...yet..
end;
destructor TRAXMesh.Destroy;
begin
     if assigned(fTexture) then fTexture.Destroy;
end;

function TRAXMesh.LoadFromFile(aFilename:string):boolean;
var
   f:Text;
   NVerts,NFaces:integer;
   Version:string;
   Dummy,MeshName,TextureName:string;
   i:cardinal;
   found:boolean;
   anObject:string;
begin
     result:=false;
     if not FileExists(aFilename) then exit;
     Assign(f,aFilename);
     Reset(f);
     ReadLn(f,Version);
     if CompareStr(Version,'1.0')<>0 then exit;
     repeat
           try
              ReadLn(f,MeshName);//Mesh name
              anObject:=MeshName;
              if UpperCase(MeshName)=(UpperCase(anObject)) then
                 found:=true
              else
                  found:=false;
              ReadLn(f,TextureName);
              if found then
              begin
                   fTexture:=TTexture.Create;
                   Delete(TextureName,1,Length(TEXTURE_INDICATOR));
                   fTexture.LoadTexture(fTextureDir+TextureName);
              end;
              ReadLn(f,Dummy);//Mesh indices indicator
              ReadLn(f,NFaces);
              if found then
                 SetLength(fFaces,NFaces);
              for i:=0 to NFaces-1 do
                  if found then
                     ReadLn(f,fFaces[i].a,fFaces[i].b,fFaces[i].c)
                  else
                      ReadLn(f,Dummy);
              ReadLn(f,Dummy);//Mesh vertices indicator
              ReadLn(f,NVerts);
              if found then
                 SetLength(fVertices,NVerts);
              if found then
                 for i:=0 to NVerts-1 do
                 begin
                     if found then ReadLn(f,fVertices[i].p[X],fVertices[i].p[Y],fVertices[i].p[Z],fVertices[i].u,fVertices[i].v)
                     else Readln(f,Dummy);
                 end;
              if found then;
                 for i:=0 to NFaces-1 do
                     fFaces[i].Normal:=GetNormal(fVertices[fFaces[i].A].p,
                                                 fVertices[fFaces[i].B].p,
                                                 fVertices[fFaces[i].C].p);
              if found then
              begin
                   Close(f);
                   result:=found;
                   exit;
              end;
           except
                 exit;
                 Close(f);
           end;
     until EOF(f);
     result:=false;
     Close(f);
end;

procedure TRAXMesh.Render(aCamera:TCamera);
var
   i:integer;
begin
     if assigned(fTexture) then
     begin
          fTexture.Enable;
          fTexture.Opaque;
     end;
     glPushMatrix;
     //glTranslatef(fTranslation[X],fTranslation[Y],fTranslation[Z]);
     //glRotatef(fRotation[X],1.0,0.0,0.0);
     //glRotatef(fRotation[Y],0.0,1.0,0.0);
     //glRotatef(fRotation[Z],0.0,0.0,1.0);
     //glScalef(fScaling[X],fScaling[Y],fScaling[Z]);
     //Apply local transform here...
     glBegin(GL_TRIANGLES);
                           for i:=0 to Length(fFaces)-1 do
                           begin
                                glTexCoord2f(fVertices[fFaces[i].A].u,fVertices[fFaces[i].A].v);
                                glVertex3fv(@fVertices[fFaces[i].A].p);

                                glTexCoord2f(fVertices[fFaces[i].B].u,fVertices[fFaces[i].B].v);
                                glVertex3fv(@fVertices[fFaces[i].B].p);

                                glTexCoord2f(fVertices[fFaces[i].C].u,fVertices[fFaces[i].C].v);
                                glVertex3fv(@fVertices[fFaces[i].C].p);
                           end;
     glEnd;
     glPopMatrix;
end;

procedure TRAXMesh.Update(TickCount: cardinal);
begin
     //nothing...
end;

{ TMesh }

constructor TMesh.Create;
begin
     fBoundingVolume:=TSphereBoundingVolume.Create;
end;

destructor TMesh.Destroy;
begin
     fBoundingVolume.Free;
end;

end.
