unit BoundingVolumes;

interface

uses
    BaseTypes;

const
     NOT_IN_FRUSTUM=0;
     PARTIALLY_IN_FRUSTUM=1;
     TOTALLY_IN_FRUSTUM=2;

type
    PBoundsRec=pointer;

    TBoundingVolume=class
                         fBoundsRec:PBoundsRec;
                         public
                         procedure UpdateBounds(aRec:PBoundsRec);virtual;abstract;
                         function Collision(aBoundingVolume:TBoundingVolume):boolean;virtual;abstract;
                         property BoundsRec:PBoundsRec
                                  read fBoundsRec
                                  write fBoundsRec;
    end;
    PSphereBoundRec=^TSphereBoundRec;
    TSphereBoundRec=record
                          Origin:TVector;
                          //This is the world position. It is upto the owner to
                          //Update with world coords.
                          //no transform for AABB's and spherical bounds
                          Radius:TFloat;
    end;
    TSphereBoundingVolume=class(TBoundingVolume)
                               public
                               constructor Create;
                               destructor Destroy;override;
                               procedure UpdateBounds(aRec:PBoundsRec);override;
                               function Collision(aBoundingVolume:TBoundingVolume):boolean;override;
    end;

implementation

uses
    BaseMath;

{ TSphereBoundingVolume }

function TSphereBoundingVolume.Collision(aBoundingVolume: TBoundingVolume): boolean;
begin
     result:=SquaredCartesianDistance(PSphereBoundRec(fBoundsRec)^.Origin,
                                      PSphereBoundRec(aBoundingVolume.BoundsRec)^.Origin)>=
             Sqr(PSphereBoundRec(fBoundsRec)^.Radius+
                 PSphereBoundRec(aBoundingVolume.BoundsRec)^.Radius);
end;

constructor TSphereBoundingVolume.Create;
begin
     GetMem(fBoundsRec,SizeOf(TSphereBoundRec));
end;

destructor TSphereBoundingVolume.Destroy;
begin
     FreeMem(fBoundsRec,SizeOf(TSphereBoundRec));
end;

procedure TSphereBoundingVolume.UpdateBounds(aRec: PBoundsRec);
begin
     PSphereBoundRec(fBoundsRec)^.Origin:=PSphereBoundRec(aRec)^.Origin;
     PSphereBoundRec(fBoundsRec)^.Radius:=PSphereBoundRec(aRec)^.Radius;
end;

end.
