unit GLBillboard;

interface

uses
    GL,
    BaseTypes,
    Base3DObject,
    BaseMath,
    Camera,
    GLTexture;

type
    TBillboard=class(TBaseObject)
                               protected
                               fColor:TColorf;
                               fSize:TFloat;
                               fTranslation:TVector;
                               fTexture:TTexture;
                               public
                               constructor Create;
                               destructor Destroy;override;
                               procedure PreRender;override;
                               procedure Render(aCamera:TCamera);override;
                               property Color:TColorf
                                        read fColor
                                        write fColor;
                               property Size:TFloat
                                        read fSize
                                        write fSize;
                               property Translation:TVector
                                        read fTranslation
                                        write fTranslation;
                               property Texture:TTexture
                                        read fTexture
                                        write fTexture;
    end;

implementation

{ TBillboard }

constructor TBillboard.Create;
begin
     inherited;
     fColor:=MakeColorf(1,1,1,0.5);
     fTexture:=TTexture.Create;
end;

destructor TBillboard.Destroy;
begin
     inherited;
     if assigned(fTexture) then fTexture.Free;
end;

procedure TBillboard.PreRender;
begin
     glPushMatrix;
end;

procedure TBillboard.Render(aCamera:TCamera);
var
   camRotation:TMatrix;
begin
     camRotation:=aCamera.GetRotation;
     glMultMatrixf(@camRotation[0,0]);
     glTranslatef(fTranslation[X],fTranslation[Y],fTranslation[Z]);
     if assigned(fTexture) then
     begin
          fTexture.Translucency;
          fTexture.Enable;
     end;
     glColor4fv(@fColor);
     glBegin(GL_QUADS);
       glTexCoord2f(0.0, 0.0); glVertex3f(-fSize, -fSize, 0);
       glTexCoord2f(1.0, 0.0); glVertex3f( fSize, -fSize, 0);
       glTexCoord2f(1.0, 1.0); glVertex3f( fSize,  fSize, 0);
       glTexCoord2f(0.0, 1.0); glVertex3f(-fSize,  fSize, 0);
     glEnd();
     glPopMatrix;
end;

end.
