unit GLConsole;

interface

uses
    GL,
    GLU,
    BaseTypes,
    GLText,
    Windows,
    Classes,
    SysUtils,
    ExtCtrls,
    GLTexture;

type
    TConsoleEvent=procedure (command:string) of object;
    TConsole=class
                  private
                         fFont:TGLFont;
                         fLines:TStringList;
                         fNumLines:integer;
                         fMaxLength:integer;
                         fCursorTimer:TTimer;
                         fCursorVisible:boolean;
                         fOnConsoleCommand:TConsoleEvent;
                         fWidth,fHeight:integer;
                         fConback:TTexture;
                         fCharset:integer;
                  protected
                  function GetVisible:boolean;
                  procedure SetVisible(val:boolean);
                  procedure CursorEvent(Sender:TObject);
                  public
                  constructor Create(aFont:TGLFont);
                  function LoadConsoleFile(afilename:string):boolean;
                  procedure Init;
                  procedure Render;
                  procedure Update(Key:char);
                  procedure Write(s:string);
                  procedure AddWrap(s:string);
                  procedure Status(s:string);
                  procedure Command(s:string);
                  destructor Destroy;override;
                  property NumLines:integer
                           read fNumLines
                           write fNumLines;
                  property Visible:boolean
                           read GetVisible
                           write SetVisible;
                  property Lines:TStringList
                           read fLines
                           write fLines;
                  property MaxLength:integer
                           read fMaxLength
                           write fMaxLength;
                  property OnConsoleCommand:TConsoleEvent
                           read fOnConsoleCommand
                           write fOnConsoleCommand;
                  property Width:integer
                           read fWidth
                           write fWidth;
                  property Height:integer
                           read fHeight
                           write fHeight;
                  property Conback:TTexture
                           read fConback
                           write fConback;
                  property Charset:integer
                           read fCharset
                           write fCharset;
    end;

implementation
function TConsole.GetVisible:boolean;
begin
     result:=fFont.Visible;
end;
procedure TConsole.Init;
begin
     fLines.Add('>');
end;
procedure TConsole.Write(s:string);
begin
     fLines.Add(s);
end;
procedure TConsole.Status(s:string);
begin
     fLines[fLines.Count-2]:=s;
end;
procedure TConsole.AddWrap(s:string);
begin
     s:=WrapText(s, #13#10, ['.',' ',#9,'-'], fMaxLength);
     fLines.Text:=fLines.Text+s;
end;
procedure TConsole.Command(s:string);
begin
     fLines[fLines.Count-1]:=fLines[fLines.Count-1]+s;
     Update(#13);
end;
procedure TConsole.SetVisible(val:boolean);
begin
     fFont.Visible:=val;
     if val then
        fCursorTimer.Enabled:=true
     else
         fCursorTimer.Enabled:=false;
end;
constructor TConsole.Create(aFont:TGLFont);
begin
     inherited Create;
     fFont:=aFont;
     fLines:=TStringList.Create;
     fNumLines:=10;
     fMaxLength:=(SCRWIDTH div FONTWIDTH)-1;
     fCursorTimer:=TTimer.Create(nil);
     fCursorTimer.Enabled:=false;
     fCursorTimer.Interval:=200;
     fCursorTimer.OnTimer:=CursorEvent;
     fConback:=TTexture.Create;
     fCharset:=0;
     fWidth:=SCRWIDTH;
     fHeight:=fNumLines*(FONTHEIGHT*2)+10;

     AddWrap('----------------------------------------------------------------------------');
     AddWrap('**Lucifer console 2.0**');
     AddWrap('Written by Ananth B. AKA HellSpawn');
     AddWrap('----------------------------------------------------------------------------');
     AddWrap('OpenGL init....OK');
     AddWrap('Console Init....OK');
     AddWrap('Font Init....OK');
     AddWrap('Camera Init....OK');
     AddWrap('Script Init....OK');
end;
procedure TConsole.CursorEvent(Sender:TObject);
begin
     fCursorVisible:=not fCursorVisible;
end;
procedure TConsole.Render;
var
   i,row:integer;
begin
     if GetVisible then
     begin
          glLoadIdentity;
          fConback.Translucency;
          fConback.Enable;
          DrawFSQuad(fWidth,fHeight);
          glPopAttrib;
          row:=0;
          glDisable(GL_DEPTH_TEST);
          if fLines.Count=0 then exit;
          for i:=fLines.Count-fNumLines to fLines.Count-1 do
          begin
               if (i=fLines.Count-1) and fCursorVisible then
                  fFont.Print(0,row,fCharset,PChar(fLines[i]+'_'))
               else
                   fFont.Print(0,row,fCharset,PChar(fLines[i]));
               Inc(row);
          end;
          glenable(GL_DEPTH_TEST);
     end;
end;
function TConsole.LoadConsoleFile(afilename:string):boolean;
begin
     result:=false;
     if not FileExists(aFilename) then exit;
     try
        fLines.LoadFromFile(aFilename);
     except
           exit;
     end;
     result:=true;
end;
destructor TConsole.Destroy;
begin
     if assigned(fLines) then fLines.Destroy;
     inherited;
end;
procedure TConsole.Update(Key:char);
var
   lastLine:integer;
   s:string;
begin
     lastLine:=fLines.Count-1;
     case Key of
          #8:begin
                  s:=fLines[lastLine];
                  if Length(s)>1 then
                  begin
                       Delete(s,Length(fLines[lastLine]),1);
                       fLines[lastLine]:=s;
                  end;
          end;
          #13:begin
                   s:=fLines[fLines.Count-1];
                   Delete(s,1,1);
                   fLines[fLines.Count-1]:=s;
                   if assigned(fOnConsoleCommand) then fOnConsoleCommand(fLines[fLines.Count-1]);
                   fLines.Add('>');
          end;
          else
              if (Length(fLines[lastLine])+1)<=fMaxLength then
                fLines[lastLine]:=fLines[lastLine]+Key;
     end;
end;
end.

