unit GLParticleSystem;

interface

uses
    Windows,
    GL,
    Classes,
    BaseTypes,
    BaseMath,
    Base3DObject,
    GLTexture,
    GLBillBoard,
    Camera;

const
     FULL_LIFE=1;
     NO_LIFE:single=0;
type
    PParticle=^TParticle;
    TParticle=record
                    Particle:TBillBoard;
                    Velocity,Acceleration:TVector;
                    Life:TFloat;
    end;
    TParticleSystem=class(T3DObject)
                    protected
                    fParticles:TList;
                    fMaxParticles:integer;
                    fTexFileName:string;
                    fInitialVelocity,fAcceleration:TVector;
                    fOrigin:TVector;
                    fParticleSize:TFloat;
                    fInitialColor:TColorf;
                    fTexture:TTexture;
                    fLifeDecrement:TFloat;
                    fAllDead:boolean;
                    procedure SetNumParticles(const Value: integer);
                    public
                    constructor Create;
                    procedure AddParticle;virtual;
                    procedure ResetParticle(i:integer);virtual;
                    procedure Render(aCamera:TCamera);override;
                    procedure PostRender;override;
                    destructor Destroy;override;
                    function LoadParticleTexture(aFilename:string):boolean;
                    function Update(TickCount:cardinal):boolean;override;
                    property NumParticles:integer
                             read fMaxParticles
                             write SetNumParticles;
                    property Origin:TVector
                             read fOrigin
                             write fOrigin;
                    property InitialVelocity:TVector
                             read fInitialVelocity
                             write fInitialVelocity;
                    property Acceleration:TVector
                             read fAcceleration
                             write fAcceleration;
                    property ParticleSize:TFloat
                             read fParticleSize
                             write fParticleSize;
                    property InitialColor:TColorf
                             read fInitialColor
                             write fInitialColor;
                    property LifeDecrement:TFloat
                             read fLifeDecrement
                             write fLifeDecrement;
                    property AllDead:boolean
                             read fAllDead;
    end;
    TFire=class(TParticleSystem)
               public
               procedure Render(aCamera:TCamera);override;               
    end;
    T3DShellExplode=class(TParticleSystem)
                         public
                         constructor Create;
                         procedure AddParticle;override;
                         procedure ResetParticle(i:integer);override;
    end;
    TExplode=class(TParticleSystem)
                  public
                  constructor Create;
                  procedure AddParticle;override;
                  procedure ResetParticle(i:integer);override;
    end;
    TShockwaveExplode=class(TParticleSystem)
                      public
                      constructor Create;
                      procedure AddParticle;override;
                      procedure ResetParticle(i:integer);override;
    end;

implementation

{ TParticleSystem }

procedure TParticleSystem.AddParticle;
var
   tempParticle:PParticle;
begin
     New(tempParticle);
     tempParticle.Particle:=TBillBoard.Create;
     tempParticle.Particle.Texture:=fTexture;
     tempParticle.Velocity:=MakeVector((random-random)/10,
                                       (random-random)/10,
                                       (random-random)/10);
     tempParticle.Life:=FULL_LIFE;
     tempParticle.Acceleration:=fAcceleration;
     tempParticle.Particle.Translation:=fOrigin;
     tempParticle.Particle.Size:=fParticleSize;
     tempParticle.Particle.Color:=fInitialColor;
     tempParticle.Particle.FrameInterval:=0;
     fParticles.Add(tempParticle);
end;
procedure TParticleSystem.ResetParticle(i:integer);
var
   tempParticle:PParticle;
begin
     tempParticle:=PParticle(fParticles.Items[i]);
     tempParticle.Velocity:=MakeVector((random-random)/10,
                                       (random-random)/10,
                                       (random-random)/10);
     tempParticle.Particle.Size:=fParticleSize;
     tempParticle.Life:=FULL_LIFE;
     tempParticle.Particle.Translation:=fOrigin;
     tempParticle.Particle.Color:=fInitialColor;
     tempParticle.Acceleration:=fAcceleration;
end;

constructor TParticleSystem.Create;
begin
     inherited;
     fParticles:=TList.Create;
     fTexture:=TTexture.Create;
     fParticleSize:=2;
     fInitialColor:=MakeColorf(0.9215,0.8,0.466,1);
     fLifeDecrement:=0.03;
     fAcceleration:=MakeVector(0,0.03,0);//up....like a fire/flame
end;

destructor TParticleSystem.Destroy;
var
   i:integer;
begin
     inherited;
     if assigned(fParticles) then
        for i:=0 to fParticles.Count-1 do
            PParticle(fParticles.Items[i])^.Particle.Free;
     fTexture.Free;
end;

function TParticleSystem.LoadParticleTexture(aFilename: string): boolean;
begin
     result:=fTexture.LoadTexture(aFilename);
     if result then
        fTexFileName:=aFilename;
end;

function TParticleSystem.Update(TickCount: cardinal): boolean;
var
   i:integer;
   temp:PParticle;
begin
     result:=inherited Update(TickCount);
     if not result then exit;
     fAllDead:=true;
     for i:=0 to fParticles.Count-1 do
     begin
          temp:=PParticle(fParticles.Items[i]);
          temp.Life:=temp.Life-fLifeDecrement;
          if temp.Life<=NO_LIFE then
          begin
               ResetParticle(i);
               exit;
          end;
          if temp.Life>NO_LIFE then
             fAllDead:=false;
          temp.Particle.Translation:=VectorAdd(temp.Particle.Translation,temp.Velocity);
          temp.Velocity:=VectorAdd(temp.Velocity,temp.Acceleration);
          fParticles.Items[i]:=temp;
     end;
end;

procedure TParticleSystem.Render(aCamera:TCamera);
var
   i:integer;
begin
     if AllDead then exit;
     glPushAttrib(GL_ALL_ATTRIB_BITS);
     glDepthMask(GL_FALSE);
     glTranslatef(fOrigin[X],fOrigin[Y],fOrigin[Z]);
     for i:=0 to fParticles.Count-1 do
     begin
          PParticle(fParticles[i])^.Particle.Color:=MakeColorf(PParticle(fParticles[i])^.Particle.Color.R,
                                                               PParticle(fParticles[i])^.Particle.Color.G,
                                                               PParticle(fParticles[i])^.Particle.Color.B,
                                                               PParticle(fParticles[i])^.Life);
          PParticle(fParticles[i])^.Particle.PreRender;
          PParticle(fParticles[i])^.Particle.Render(aCamera);
     end;
     glPopAttrib;
     glDepthMask(GL_TRUE);
end;

procedure TParticleSystem.SetNumParticles(const Value: integer);
var
   i:integer;
begin
     fParticles.Clear;
     fMaxParticles:=Value;
     for i:=0 to fMaxParticles-1 do
         AddParticle;
end;

procedure TParticleSystem.PostRender;
begin
     //nothing...
end;

{ T3DShellExplode }

procedure T3DShellExplode.AddParticle;
var
   tempParticle:PParticle;
begin
     New(tempParticle);
     tempParticle.Particle:=TBillBoard.Create;
     tempParticle.Particle.Texture:=fTexture;
     tempParticle.Velocity:=MakeVector((random-random),
                                       (random-random),
                                       (random-random));
     tempParticle.Velocity:=VectorNormalize(tempParticle.Velocity);
     tempParticle.Life:=FULL_LIFE;
     tempParticle.Acceleration:=fAcceleration;
     tempParticle.Particle.Translation:=fOrigin;
     tempParticle.Particle.Size:=fParticleSize;
     tempParticle.Particle.Color:=fInitialColor;
     fParticles.Add(tempParticle);
end;

constructor T3DShellExplode.Create;
begin
     inherited;
     fParticleSize:=1;
     fLifeDecrement:=0.1;
     FrameInterval:=50;
     fAcceleration:=MakeVector(0.001,0.001,0.001);
end;

procedure T3DShellExplode.ResetParticle(i: integer);
begin
     //Do nothing. The explosion dies out.
end;

{ TFire }

procedure TFire.Render(aCamera: TCamera);
var
   i:integer;
begin
     glPushAttrib(GL_ALL_ATTRIB_BITS);
     glDepthMask(GL_FALSE);
     glTranslatef(fOrigin[X],fOrigin[Y],fOrigin[Z]);
     glPushMatrix;
     for i:=0 to fParticles.Count-1 do
     begin
          PParticle(fParticles[i])^.Particle.Color:=MakeColorf(PParticle(fParticles[i])^.Particle.Color.R,
                                                               PParticle(fParticles[i])^.Particle.Color.G,
                                                               PParticle(fParticles[i])^.Particle.Color.B,
                                                               PParticle(fParticles[i])^.Life);
          PParticle(fParticles[i])^.Particle.PreRender;
          PParticle(fParticles[i])^.Particle.Render(aCamera);
     end;
     glPopAttrib;
     glDepthMask(GL_TRUE);
     glPopMatrix;
end;

{ TExplode }

procedure TExplode.AddParticle;
var
   tempParticle:PParticle;
begin
     New(tempParticle);
     tempParticle.Particle:=TBillBoard.Create;
     tempParticle.Particle.Texture:=fTexture;
     tempParticle.Velocity:=MakeVector((random-random),
                                       (random-random),
                                       (random-random));
     //tempParticle.Velocity:=VectorNormalize(tempParticle.Velocity);
     tempParticle.Life:=FULL_LIFE;
     tempParticle.Acceleration:=fAcceleration;
     tempParticle.Particle.Translation:=fOrigin;
     tempParticle.Particle.Size:=fParticleSize;
     tempParticle.Particle.Color:=fInitialColor;
     fParticles.Add(tempParticle);
end;

constructor TExplode.Create;
begin
     inherited;
     fParticleSize:=1;
     fLifeDecrement:=0.1;
     FrameInterval:=50;
     fAcceleration:=MakeVector(0.001,0.001,0.001);
end;

procedure TExplode.ResetParticle(i: integer);
begin
     //Do nothing. The explosion dies out.
end;

{ TShockwaveExplode }

procedure TShockwaveExplode.AddParticle;
var
   tempParticle:PParticle;
begin
     New(tempParticle);
     tempParticle.Particle:=TBillBoard.Create;
     tempParticle.Particle.Texture:=fTexture;
     tempParticle.Velocity:=MakeVector((random-random),
                                       (random-random),
                                       (random-random));
     tempParticle.Velocity:=VectorNormalize(tempParticle.Velocity);
     tempParticle.Velocity[Y]:=0;
     tempParticle.Life:=FULL_LIFE;
     tempParticle.Acceleration:=fAcceleration;
     tempParticle.Particle.Translation:=fOrigin;
     tempParticle.Particle.Size:=fParticleSize;
     tempParticle.Particle.Color:=fInitialColor;
     fParticles.Add(tempParticle);
end;

constructor TShockwaveExplode.Create;
begin
     inherited;
     fParticleSize:=1;
     fLifeDecrement:=0.1;
     FrameInterval:=50;
     fAcceleration:=MakeVector(0.001,0.001,0.001);
end;

procedure TShockwaveExplode.ResetParticle(i: integer);
begin
     //nothing...the shockwave dies out
end;

end.
