unit GLText;

interface

uses
    GL,
    GLU,
    BaseTypes,
    BaseMath,
    GLTexture,
    Base3DObject,
    Camera,
    Windows,
    SysUtils;

const
     SCRWIDTH:integer=640;
     SCRHEIGHT:integer=480;
     FONTWIDTH:integer=16;
     FONTHEIGHT:integer=24;

type
    TGLFont=class
               private
                      fTexture:TTexture;
                      fBase:GLuint;
                      fColor:TColorb;
                      fFadeDone,fVisible:boolean;
                      fFrameInterval:cardinal;
                      fLastTickCount:cardinal;
                      fTextModePlaceMent:boolean;
               protected
               procedure SetVisible(val:boolean);
               public
               constructor CreateFromTGA(aTGA:string);
               constructor CreateFromTexture(aTexture:TTexture);
               function Update(TickCount:cardinal):boolean;virtual;
               procedure Print(x,y:GLint;fontset:cardinal;text:pchar);
               destructor Destroy;override;
               procedure Fadeout;
               procedure FadeIn;
               property Texture:TTexture
                        read fTexture
                        write fTexture;
               property Color:TColorb
                        read fColor
                        write fColor;
               property Visible:boolean
                        read fVisible
                        write SetVisible;
               property R:byte
                        read fColor.R
                        write fColor.R default 255;
               property G:byte
                        read fColor.G
                        write fColor.G default 255;
               property B:byte
                        read fColor.B
                        write fColor.B default 255;
               property A:byte
                        read fColor.A
                        write fColor.A default 255;
               property FrameInterval:cardinal
                        read fFrameInterval
                        write fFrameInterval default 50;
               property TextModePlacement:boolean
                        read fTextModePlacement
                        write fTextModePlacement;
    end;
    TTextObject=class(TBaseObject)
                     private
                     fStartPos,fEndPos,fCurrPos:T2DVector;
                     fVelocity,fAcceleration:T2DVector;
                     fFont:TGLFont;
                     fDisplayText:string;
                     fCharset:integer;
                     fLambda:TFloat;
                     fDone:boolean;
                     procedure SetStartPos(const Value: T2DVector);
                     protected
                     public
                     function Update(TickCount:cardinal):boolean;override;
                     constructor Create(aTexture:TTexture);
                     procedure PreRender;override;
                     procedure Render(aCamera:TCamera);override;
                     procedure PostRender;override;
                     property StartPos:T2DVector
                              read fStartPos
                              write SetStartPos;
                     property EndPos:T2DVector
                              read fEndPos
                              write fEndPos;
                     property CurrPos:T2DVector
                              read fCurrPos;
                     property Velocity:T2DVector
                              read fVelocity
                              write fVelocity;
                     property Acceleration:T2DVector
                              read fAcceleration
                              write fAcceleration;
                     property Font:TGLFont
                              read fFont
                              write fFont;
                     property DisplayText:string
                              read fDisplayText
                              write fDisplayText;
                     property Charset:integer
                              read fCharset
                              write fCharset;
                     property Done:boolean
                              read fDone;
                     property Lambda:TFloat
                              read fLambda;
    end;
    
procedure Setup2DScreen(aWidth,aHeight:integer);
procedure DrawFSQuad(aWidth,aHeight:integer);
implementation

procedure Setup2DScreen(aWidth,aHeight:integer);
begin
     glViewport(0,0,aWidth,aHeight);
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity;
     glOrtho(0.0,aWidth,aHeight,0.0,-1.0,1.0);
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity;
end;

procedure DrawFSQuad(aWidth,aHeight:integer);
begin
     glBegin(GL_QUADS);
            glTexCoord2f(0.0, 0.0);
            glVertex2d( 0,aHeight);
            glTexCoord2f(1.0, 0.0);
            glVertex2d(aWidth,aHeight);
            glTexCoord2f(1.0, 1.0);
            glVertex2d(aWidth, 0);
            glTexCoord2f(0.0, 1.0);
            glVertex2d( 0, 0);
     glEnd;
end;

procedure TGLFont.SetVisible(val:boolean);
begin
     fVisible:=val;
     fFadeDone:=false;
end;
function TGLFont.Update(TickCount:cardinal):boolean;
begin
     if (TickCount-fLastTickCount)>=fFrameInterval then
        result:=true
     else
         result:=false;
     fLastTickCount:=TickCount;
end;
procedure TGLFont.Fadeout;
begin
     if not update(GetTickCount) then exit;
     if (fColor.A=0) then
        fFadeDone:=true
     else
         Dec(fColor.A);
end;
procedure TGLFont.FadeIn;
begin
     if not update(GetTickCount) then exit;
     if (fColor.A=127) then
        fFadeDone:=true
     else
         Inc(fColor.A);
end;
constructor TGLFont.CreateFromTGA(aTGA:string);
var
   cx,cy:single;
   loop1:cardinal;
   width,height:integer;
   space:integer;
begin
     width:=SCRWIDTH*16 div 640;
     height:=SCRHEIGHT*16 div 480;
     space:=SCRWIDTH*14 div 640;
     FONTHEIGHT:=height;
     FONTWIDTH:=width;
     inherited Create;
     fTexture:=TTexture.Create;
     fTexture.MIPMapping:=false;
     fTextModePlaceMent:=false;
     fFrameInterval:=1;
     if not fTexture.LoadTGA(aTGA) then
        raise exception.Create('Could not load font texture');
     fbase := glGenLists(256);
     glBindTexture(GL_TEXTURE_2D, fTexture.ID);
     for loop1 := 0 to 255 do
     begin
          cx := (loop1 mod 16) / 16;
          cy := (loop1 div 16) / 16;

          glNewList(fbase+loop1,GL_COMPILE);
              glBegin(GL_QUADS);
                  glTexCoord2f(cx, 1.0 - cy - 0.0625);
                  glVertex2d( 0,height);
                  glTexCoord2f(cx + 0.0625, 1.0 - cy - 0.0625);
                  glVertex2d(width,height);
                  glTexCoord2f(cx + 0.0625, 1.0 - cy);
                  glVertex2d(width, 0);
                  glTexCoord2f(cx, 1.0 - cy);
                  glVertex2d( 0, 0);
              glEnd;
              glTranslated(space,0,0);
          glEndList;
     end;
     fColor:=MakeColorb(127,127,127,127);
     fFrameInterval:=5;
end;
procedure TGLFont.Print(x, y : GLint; fontset : cardinal; text : pchar);
begin
     glLoadIdentity;
     if not Visible and fFadeDone then exit;
     if not fFadeDone then
        case fVisible of
             true:FadeIn;
             false:FadeOut;
        end;
     if (fontset > 1) then fontset := 1;
     fTexture.Enable;
     glColor4bv(@fColor);
     fTexture.Transparency;
     if fTextModePlaceMent then
        glTranslated(x*FONTWIDTH,y*(FONTHEIGHT*2),0)
     else
         glTranslated(x,y,0);

     glListBase(fbase-32+(128*fontset));
     glScalef(1,2.0,1.0);
     glCallLists(strlen(text),GL_UNSIGNED_BYTE, text);

     glColor4b(127,127,127,127);
end;
destructor TGLFont.Destroy;
begin
     fTexture.Free;
     glDeleteLists(fBase,256);
     inherited Destroy;
end;
constructor TGLFont.CreateFromTexture(aTexture: TTexture);
var
   cx,cy:single;
   loop1:cardinal;
   width,height:integer;
   space:integer;
begin
     width:=SCRWIDTH*16 div 640;
     height:=SCRHEIGHT*16 div 480;
     space:=SCRWIDTH*14 div 640;
     FONTHEIGHT:=height;
     FONTWIDTH:=width;
     inherited Create;
     fTexture:=aTexture;
     fTextModePlaceMent:=false;
     fbase := glGenLists(256);
     glBindTexture(GL_TEXTURE_2D, fTexture.ID);
     for loop1 := 0 to 255 do
     begin
          cx := (loop1 mod 16) / 16;
          cy := (loop1 div 16) / 16;

          glNewList(fbase+loop1,GL_COMPILE);
              glBegin(GL_QUADS);
                  glTexCoord2f(cx, 1.0 - cy - 0.0625);
                  glVertex2d( 0,height);
                  glTexCoord2f(cx + 0.0625, 1.0 - cy - 0.0625);
                  glVertex2d(width,height);
                  glTexCoord2f(cx + 0.0625, 1.0 - cy);
                  glVertex2d(width, 0);
                  glTexCoord2f(cx, 1.0 - cy);
                  glVertex2d( 0, 0);
              glEnd;
              glTranslated(space,0,0);
          glEndList;
     end;
     fColor:=MakeColorb(127,127,127,127);
     fFrameInterval:=5;
end;

{ TTextObject }

constructor TTextObject.Create(aTexture: TTexture);
begin
     inherited Create;
     fFont:=TGLFont.CreateFromTexture(aTexture);
     fFont.Visible:=true;
     fFont.Color:=MakeColorb(127,127,127,0);
     fFont.TextModePlacement:=false;
     fCharset:=0;
     fVelocity:=Make2DVector(0,0);
     fAcceleration:=Make2DVector(0,0);
     fStartPos:=Make2DVector(0,0);
     FrameInterval:=5;
end;

procedure TTextObject.PostRender;
begin
end;

procedure TTextObject.PreRender;
begin
     Update(GetTickCount);
end;

procedure TTextObject.Render;
begin
     fFont.Print(Round(fCurrPos[0]),Round(fCurrPos[1]),fCharset,PChar(fDisplayText));
end;

procedure TTextObject.SetStartPos(const Value: T2DVector);
begin
     fStartPos:=Value;
     fCurrPos:=fStartPos;
     fLambda:=0;
     fDone:=false;
end;

function TTextObject.Update(TickCount: cardinal):boolean;
begin
     result:=inherited Update(TickCount);
     if not result then exit;
     if VectorEqual(fCurrPos,fEndPos) then
     begin
          fDone:=true;
          result:=false;
          exit;
     end;
     fCurrPos:=Interpolate(fStartPos,fEndPos,fLambda);
     fLambda:=fLambda+LAMBDA_INCREMENT;
     if fLambda<0.5 then
        fFont.Visible:=true
     else
         fFont.Visible:=false;
end;

end.
