unit GLTimer;

interface

{$B-,J+,Q-,R-,T-,X+}

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Controls,
  Forms,
  MMSystem,
  Math;

type

  TGLTimerEvent = procedure(Sender: TObject; LagCount: Integer) of object;

  TCustomGLTimer = class(TComponent)
  private
    FActiveOnly: Boolean;
    FEnabled: Boolean;
    FFrameRate: Integer;
    FInitialized: Boolean;
    FInterval: Cardinal;
    FInterval2: Cardinal;
    FNowFrameRate: Integer;
    FOldTime: DWORD;
    FOldTime2: DWORD;
    FOnActivate: TNotifyEvent;
    FOnDeactivate: TNotifyEvent;
    FOnTimer: TGLTimerEvent;
    procedure AppIdle(Sender: TObject; var Done: Boolean);
    function AppProc(var Message: TMessage): Boolean;
    procedure Finalize;
    procedure Initialize;
    procedure Resume;
    procedure SetActiveOnly(Value: Boolean);
    procedure SetEnabled(Value: Boolean);
    procedure SetInterval(Value: Cardinal);
    procedure Suspend;
  protected
    procedure DoActivate; virtual;
    procedure DoDeactivate; virtual;
    procedure DoTimer(LagCount: Integer); virtual;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property ActiveOnly: Boolean read FActiveOnly write SetActiveOnly;
    property Enabled: Boolean read FEnabled write SetEnabled;
    property FrameRate: Integer read FFrameRate;
    property Interval: Cardinal read FInterval write SetInterval;
    property OnActivate: TNotifyEvent read FOnActivate write FOnActivate;
    property OnDeactivate: TNotifyEvent read FOnDeactivate write FOnDeactivate;
    property OnTimer: TGLTimerEvent read FOnTimer write FOnTimer;
  end;

  {  TGLTimer  }

  TGLTimer = class(TCustomGLTimer)
  published
    property ActiveOnly;
    property Enabled;
    property Interval;
    property OnActivate;
    property OnDeactivate;
    property OnTimer;
  end;

implementation
{  TCustomGLTimer  }

constructor TCustomGLTimer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FActiveOnly := True;
  FEnabled := True;
  Interval := 1000;
  Application.HookMainWindow(AppProc);
end;

destructor TCustomGLTimer.Destroy;
begin
  Finalize;
  Application.UnHookMainWindow(AppProc);
  inherited Destroy;
end;

procedure TCustomGLTimer.AppIdle(Sender: TObject; var Done: Boolean);
var
  t, t2: DWORD;
  LagCount, i: Integer;
begin
  Done := False;

  t := TimeGetTime;
  t2 := t - FOldTime;
  if t2 >= FInterval then
  begin
    FOldTime := t;

    LagCount := t2 div FInterval2;
    if LagCount < 1 then
      LagCount := 1;

    Inc(FNowFrameRate);

    i := Max(t - FOldTime2, 1);
    if i >= 1000 then
    begin
      FFrameRate := Round(FNowFrameRate * 1000 / i);
      FNowFrameRate := 0;
      FOldTime2 := t;
    end;

    DoTimer(LagCount);
  end;
end;

function TCustomGLTimer.AppProc(var Message: TMessage): Boolean;
begin
  Result := False;
  case Message.Msg of
    CM_ACTIVATE:
      begin
        DoActivate;
        if FInitialized and FActiveOnly then
          Resume;
      end;
    CM_DEACTIVATE:
      begin
        DoDeactivate;
        if FInitialized and FActiveOnly then
          Suspend;
      end;
  end;
end;

procedure TCustomGLTimer.DoActivate;
begin
  if Assigned(FOnActivate) then
    FOnActivate(Self);
end;

procedure TCustomGLTimer.DoDeactivate;
begin
  if Assigned(FOnDeactivate) then
    FOnDeactivate(Self);
end;

procedure TCustomGLTimer.DoTimer(LagCount: Integer);
begin
  if Assigned(FOnTimer) then
    FOnTimer(Self, LagCount);
end;

procedure TCustomGLTimer.Finalize;
begin
  if FInitialized then
  begin
    Suspend;
    FInitialized := False;
  end;
end;

procedure TCustomGLTimer.Initialize;
begin
  Finalize;

  if ActiveOnly then
  begin
    if Application.Active then
      Resume;
  end
  else
    Resume;
  FInitialized := True;
end;

procedure TCustomGLTimer.Loaded;
begin
  inherited Loaded;
  if (not (csDesigning in ComponentState)) and FEnabled then
    Initialize;
end;

procedure TCustomGLTimer.Resume;
begin
  FOldTime := TimeGetTime;
  FOldTime2 := TimeGetTime;
  Application.OnIdle := AppIdle;
end;

procedure TCustomGLTimer.SetActiveOnly(Value: Boolean);
begin
  if FActiveOnly <> Value then
  begin
    FActiveOnly := Value;

    if Application.Active and FActiveOnly then
      if FInitialized and FActiveOnly then
        Suspend;
  end;
end;

procedure TCustomGLTimer.SetEnabled(Value: Boolean);
begin
  if FEnabled <> Value then
  begin
    FEnabled := Value;
    if ComponentState * [csReading, csLoading] = [] then
      if FEnabled then
        Initialize
      else
        Finalize;
  end;
end;

procedure TCustomGLTimer.SetInterval(Value: Cardinal);
begin
  if FInterval <> Value then
  begin
    FInterval := Max(Value, 0);
    FInterval2 := Max(Value, 1);
  end;
end;

procedure TCustomGLTimer.Suspend;
begin
  Application.OnIdle := nil;
end;

end.

