unit MD2Mesh;

interface

uses
    GL,
    BaseTypes,
    BaseMath,
    BaseMesh,
    GLTexture,
    Windows,
    Classes,
    Camera;

const
     LAMBDA_INCREMENT:single=0.1;
     MD2_ERROR_MESSAGE:string='NO ERROR';

     //animation sequences
     IDLE1_START=0;
     IDLE1_END=39;
     RUN_START=40;
     RUN_END=45;
     SHOOT_STAND_START=46;
     SHOOT_STAND_END=53;
     SHOT_STAND_START=54;
     SHOT_STAND_END=60;
     SHOT_SHOULDER_START=61;
     SHOT_SHOULDER_END=66;
     JUMP_START=67;
     JUMP_END=71;
     IDLE2_START=72;
     IDLE2_END=95;
     SHOT_FALLDOWN_START=96;
     SHOT_FALLDOWN_END=112;
     IDLE3_START=113;
     IDLE3_END=122;
     IDLE4_START=123;
     IDLE4_END=135;
     CROUCH_START=136;
     CROUCH_END=153;
     CROUCH_CRAWL_START=154;
     CROUCH_CRAWL_END=159;
     CROUCH_SHOOT_START=160;
     CROUCH_SHOOT_END=164;
     CROUCH_IDLE_START=165;
     CROUCH_IDLE_END=169;
     CROUCH_DEATH_START=170;
     CROUCH_DEATH_END=177;
     DEATH_FALLBACK_START=178;
     DEATH_FALLBACK_END=185;
     DEATH_FALLFORWARD_START=186;
     DEATH_FALLFORWARD_END=190;
     DEATH_FALLBACKSLOW_START=191;
     DEATH_FALLBACKSLOW_END=198;

const
     maIDLE1=0;
     maRUN=1;
     maSHOT_STAND=2;
     maSHOT_SHOULDER=3;
     maJUMP=4;
     maIDLE2=5;
     maSHOT_FALLDOWN=6;
     maIDLE3=7;
     maIDLE4=8;
     maCROUCH=9;
     maCROUCH_CRAWL=10;
     maCROUCH_IDLE=11;
     maCROUCH_DEATH=12;
     maDEATH_FALLBACK=13;
     maDEATH_FALLFORWARD=14;
     maDEATH_FALLBACKSLOW=15;
     maSHOOT_STAND=16;
     maCROUCH_SHOOT=17;

     //Reversed sequences
     maRUN_BACK=18;
     maCROUCH_BACK=19;

type
    THeader=record
                 Ident:integer;
                 Version:integer;

                 SkinWidth:integer;
                 SkinHeight:integer;
                 FrameSize:integer;

                 SkinCount:integer;
                 VertexCount:integer;
                 STCount:integer;
                 FaceCount:integer;
                 GLCmdCount:integer;
                 FrameCount:integer;

                 SkinOfs:integer;
                 STOfs:integer;
                 FaceOfs:integer;
                 FrameOfs:integer;
                 GLCmdOfs:integer;
                 EndOfFile:integer;
    end;

    TSkinName =array[0..63] of char;
    TSkinNames=array[0..31] of TSkinName;

    T_ST=record
               s,t:smallint
    end;
    TSTs=array[0..2047] of T_ST;

    TFace=record
                x,y,z:smallint;
                a,b,c:smallint;
    end;
    TFaces=array[0..4095] of TFace;

    TVertex=record
                  x,y,z:byte;
                  light:byte;
    end;
    TFrame=record
                 Scale:record x,y,z:single end;
                 Trans:record x,y,z:single end;
                 Name:array[0..15] of char;
                 Verts:array[0..2047] of TVertex;
    end;
    TSequenceEndEvent=TNotifyEvent;
    TSequenceTerminateEvent=TNotifyEvent;
    TFrameEvent=procedure(FrameNum:integer) of object;
    TMD2Mesh=class(TMesh)
                        private
                                fName:string;
                                hFile:THandle;
                                hMap:THandle;
                                hPtr:pointer;
                                header:^THeader;
                                Skins:^TSkinNames;
                                STs:^TSTS;
                                Faces:^TFaces;
                                Frames:^TFrame;
                                fCurrentFrame:integer;
                                fSequence:integer;
                                fStartFrame,fEndFrame:integer;
                                fOnSequenceEnd:TSequenceEndEvent;
                                fLambda:single;
                                fVisible:boolean;
                                fLastFrame:integer;
                                fAnimationOffset:integer;
                                fOnFrame:TFrameEvent;
                                fOnSequenceTerminate:TNotifyEvent;
                                fLambdaInterval:cardinal;
                                fLastLambdaTicks:cardinal;
                                fSequenceStarted:boolean;
                                fFrameIncrement:integer;
                                fTexture:TTexture;
                         protected
                         function GetFrameCount:integer;
                         function GetFrameName(frame:integer):string;
                         procedure GetPoint(frame,index:integer;var point:TVector);
                         function LambdaUpdate(TickCount:cardinal):boolean;virtual;
                         public
                         procedure SetSequence(asequence:integer);virtual;
                         procedure Update(TickCount:cardinal);override;
                         function LoadFromFile(aFilename:string):boolean;override;
                         function LoadSkin(afilename:string):boolean;
                         procedure Render(aCamera:TCamera);override;
                         constructor Create;override;
                         destructor Destroy;override;
                         property CurrentFrame:integer
                                  read fCurrentFrame
                                  write fCurrentFrame;
                         property FrameCount:integer
                                  read GetFrameCount;
                         property FrameName[i:integer]:string
                                  read GetFrameName;
                         property Sequence:integer
                                  read fSequence
                                  write SetSequence;
                         property Visible:boolean
                                  read fVisible
                                  write fVisible;
                         property AnimationOffset:integer
                                  read fAnimationOffset
                                  write fAnimationOffset;
                         property LambdaTicks:cardinal
                                  read fLambdaInterval
                                  write fLambdaInterval;
                         property OnSequenceEnd:TSequenceEndEvent
                                  read fOnSequenceEnd
                                  write fOnSequenceEnd;
                         property OnFrame:TFrameEvent
                                  read fOnFrame
                                  write fOnFrame;
                         property OnSequenceTerminate:TNotifyEvent
                                  read fOnSequenceTerminate
                                  write fOnSequenceTerminate;
                         property FrameIncrement:integer
                                  read fFrameIncrement
                                  write fFrameIncrement;
    end;


implementation

function TMD2Mesh.GetFrameCount:integer;
begin
     result:=header.FrameCount;
end;
function TMD2Mesh.GetFrameName(frame:integer):string;
var
   F:^TFrame;
begin
     F:=pointer(integer(Frames)+frame*Header.FrameSize);
     result:=F^.Name;
end;
procedure TMD2Mesh.SetSequence(asequence:integer);
begin
     fSequence:=asequence;
     case fSequence of
          maIDLE1:begin
                       fStartFrame:=IDLE1_START+fAnimationOffset;
                       fEndFrame:=IDLE1_END+fAnimationOffset;
                       fFrameIncrement:=1;
          end;
          maRUN:begin
                     fStartFrame:=RUN_START+fAnimationOffset;
                     fEndFrame:=RUN_END+fAnimationOffset;
                     fFrameIncrement:=1;
          end;
          maSHOOT_STAND:begin
                            fStartFrame:=SHOOT_STAND_START+fAnimationOffset;
                            fEndFrame:=SHOOT_STAND_END+fAnimationOffset;
                            fFrameIncrement:=1;
          end;
          maSHOT_STAND:begin
                            fStartFrame:=SHOT_STAND_START+fAnimationOffset;
                            fEndFrame:=SHOT_STAND_END+fAnimationOffset;
                            fFrameIncrement:=1;
          end;
          maSHOT_SHOULDER:begin
                               fStartFrame:=SHOT_SHOULDER_START+fAnimationOffset;
                               fEndFrame:=SHOT_SHOULDER_END+fAnimationOffset;
                               fFrameIncrement:=1;
          end;
          maJUMP:begin
                       fStartFrame:=JUMP_START+fAnimationOffset;
                       fEndFrame:=JUMP_END+fAnimationOffset;
                       fFrameIncrement:=1;
          end;
          maIDLE2:begin
                       fStartFrame:=IDLE2_START+fAnimationOffset;
                       fEndFrame:=IDLE2_END+fAnimationOffset;
                       fFrameIncrement:=1;
          end;
          maSHOT_FALLDOWN:begin
                       fStartFrame:=SHOT_FALLDOWN_START+fAnimationOffset;
                       fEndFrame:=SHOT_FALLDOWN_END+fAnimationOffset;
                       fFrameIncrement:=1;
          end;
          maIDLE3:begin
                       fStartFrame:=IDLE3_START+fAnimationOffset;
                       fEndFrame:=IDLE3_END+fAnimationOffset;
                       fFrameIncrement:=1;
          end;
          maIDLE4:begin
                       fStartFrame:=IDLE4_START+fAnimationOffset;
                       fEndFrame:=IDLE4_END+fAnimationOffset;
                       fFrameIncrement:=1;
          end;
          maCROUCH:begin
                       fStartFrame:=CROUCH_START+fAnimationOffset;
                       fEndFrame:=CROUCH_END+fAnimationOffset;
                       fFrameIncrement:=1;
          end;
          maCROUCH_CRAWL:begin
                       fStartFrame:=CROUCH_CRAWL_START+fAnimationOffset;
                       fEndFrame:=CROUCH_CRAWL_END+fAnimationOffset;
                       fFrameIncrement:=1;
          end;
          maCROUCH_SHOOT:begin
                       fStartFrame:=CROUCH_SHOOT_START+fAnimationOffset;
                       fEndFrame:=CROUCH_SHOOT_END+fAnimationOffset;
                       fFrameIncrement:=1;
          end;
          maCROUCH_IDLE:begin
                       fStartFrame:=CROUCH_IDLE_START+fAnimationOffset;
                       fEndFrame:=CROUCH_IDLE_END+fAnimationOffset;
                       fFrameIncrement:=1;
          end;
          maCROUCH_DEATH:begin
                       fStartFrame:=CROUCH_DEATH_START+fAnimationOffset;
                       fEndFrame:=CROUCH_DEATH_END+fAnimationOffset;
                       fFrameIncrement:=1;
          end;
          maDEATH_FALLBACK:begin
                       fStartFrame:=DEATH_FALLBACK_START+fAnimationOffset;
                       fEndFrame:=DEATH_FALLBACK_END+fAnimationOffset;
                       fFrameIncrement:=1;
          end;
          maDEATH_FALLFORWARD:begin
                       fStartFrame:=DEATH_FALLFORWARD_START+fAnimationOffset;
                       fEndFrame:=DEATH_FALLFORWARD_END+fAnimationOffset;
                       fFrameIncrement:=1;
          end;
          maDEATH_FALLBACKSLOW:begin
                       fStartFrame:=DEATH_FALLBACKSLOW_START+fAnimationOffset;
                       fEndFrame:=DEATH_FALLBACKSLOW_END+fAnimationOffset;
                       fFrameIncrement:=1;
          end;
          maRUN_BACK:begin
                     fStartFrame:=RUN_END+fAnimationOffset;
                     fEndFrame:=RUN_START+fAnimationOffset;
                     fFrameIncrement:=-1;
          end;
     end;
     fCurrentFrame:=fStartFrame;
     fLambda:=0;
     if fSequenceStarted then
        if assigned(fOnSequenceTerminate) then fOnSequenceTerminate(Self);
     fSequenceStarted:=true;
end;
function TMD2Mesh.LambdaUpdate(TickCount:cardinal):boolean;
begin
     result:=false;
     if (TickCount-fLastLambdaTicks)>=fLambdaInterval then
     begin
          fLastLambdaTicks:=TickCount;
          result:=true;
     end;
end;
procedure TMD2Mesh.Update(TickCount:cardinal);
var
   lupdate:boolean;
begin
     lupdate:=LambdaUpdate(TickCount);
     if (fLambda<=1.0) and lupdate then
     begin
          fLambda:=fLambda+LAMBDA_INCREMENT;
          exit;
     end;
     if fLambda>=1.0 then
        if (fCurrentFrame)=fEndFrame  then
        begin
             fSequenceStarted:=false;
             if assigned(fOnSequenceEnd) then fOnSequenceEnd(Self);
             fLastFrame:=fCurrentFrame;
             fCurrentFrame:=fStartFrame;
             fSequenceStarted:=true;
             fLambda:=0.0;
             if assigned(fOnFrame) then
                fOnFrame(fCurrentFrame);
        end
        else
            begin
                 fLastFrame:=fCurrentFrame;
                 Inc(fCurrentFrame,fFrameIncrement);
                 fLambda:=0.0;
                 if assigned(fOnFrame) then
                    fOnFrame(fCurrentFrame);
            end
     else
         if lupdate then fLambda:=fLambda+LAMBDA_INCREMENT;
end;
function TMD2Mesh.LoadFromFile(aFilename:string):boolean;
begin
     //map the file to memory
     result:=false;
     fName:=aFileName;
     fFrameIncrement:=1;
     hFile:=CreateFile(PChar(fName),GENERIC_READ,FILE_SHARE_READ,nil,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
     if hFile=INVALID_HANDLE_VALUE then
     begin
          MD2_ERROR_MESSAGE:='Unable to open file '+fName;
          exit;
     end;
     hMap :=CreateFileMapping(hFile,nil,PAGE_READONLY,0,0,nil);
     if hMap=0 then
     begin
          MD2_ERROR_MESSAGE:='Unable to Map file '+fName;
          exit;
     end;
     hPtr:=MapViewOfFile(hMap,FILE_MAP_READ,0,0,0);
     if hPtr=nil then
     begin
          MD2_ERROR_MESSAGE:='Unable to view file '+fName;
          exit;
     end;
     header:=hPtr;
     Skins:=pointer(integer(hPtr)+header.skinofs);
     STs:=pointer(integer(hPtr)+header.STOfs);
     Faces:=pointer(integer(hPtr)+header.FaceOfs);
     Frames:=pointer(integer(hPtr)+header.FrameOfs);
     SetSequence(maRUN);
     result:=true;
     fLastFrame:=-1;
     fAnimationOffset:=0;
     fLambdaInterval:=10;
     fSequenceStarted:=false;
end;
function TMD2Mesh.LoadSkin(afilename:string):boolean;
begin
     fTexture:=TTexture.Create;
     result:=fTexture.LoadTexture(afilename);
end;
Procedure TMD2Mesh.GetPoint(frame,index:integer;var point:TVector);
var
   F:^TFrame;
begin
     F:=pointer(integer(Frames)+frame*Header.FrameSize);
     with F^ do
     begin
          point[x]:=Verts[index].x*Scale.x+Trans.x;
          point[y]:=Verts[index].y*Scale.y+Trans.y;
          point[z]:=Verts[index].z*Scale.z+Trans.z;
          point[W]:=1;
     end;
end;
procedure TMD2Mesh.Render(aCamera:TCamera);
var
   i:integer;
   p1,p2,p3:TVector;
   op1,op2,op3:TVector;
begin
     fTexture.Opaque;
     fTexture.Enable;
     glPushMatrix;

     glScalef(1/50,1/50,1/50);
     glRotatef(-90,1.0,0.0,0.0);
     glRotatef(90,0.0,0.0,1.0);

     glBegin(GL_TRIANGLES);
                           for i:=0 to header.FaceCount-1 do
                           begin
                                if fLastFrame<>-1 then
                                begin
                                     GetPoint(fLastFrame,Faces[i].x,op1);
                                     GetPoint(fLastFrame,Faces[i].y,op2);
                                     GetPoint(fLastFrame,Faces[i].z,op3);

                                     GetPoint(fCurrentFrame,Faces[i].x,p1);
                                     GetPoint(fCurrentFrame,Faces[i].y,p2);
                                     GetPoint(fCurrentFrame,Faces[i].z,p3);

                                     p1:=Interpolate(op1,p1,fLambda);
                                     p2:=Interpolate(op2,p2,fLambda);
                                     p3:=Interpolate(op3,p3,fLambda);
                                end
                                else
                                    begin
                                         GetPoint(fCurrentFrame,Faces[i].x,p1);
                                         GetPoint(fCurrentFrame,Faces[i].y,p2);
                                         GetPoint(fCurrentFrame,Faces[i].z,p3);
                                    end;

                                glTexCoord2f(STs[Faces[i].a].s/header.SkinWidth,STs[Faces[i].a].t/header.SkinHeight);
                                glVertex3fv(@p1);

                                glTexCoord2f(STs[Faces[i].b].s/header.SkinWidth,STs[Faces[i].b].t/header.SkinHeight);
                                glVertex3fv(@p2);

                                glTexCoord2f(STs[Faces[i].c].s/header.SkinWidth,STs[Faces[i].c].t/header.SkinHeight);
                                glVertex3fv(@p3);
                           end;
     glEnd;
     glPopMatrix;
end;
destructor TMD2Mesh.destroy;
begin
     inherited;
     UnMapViewOfFile(hPtr);
     CloseHandle(hMap);
     CloseHandle(hFile);
     fTexture.Free;
end;

constructor TMD2Mesh.Create;
begin
     SetSequence(maIDLE1);
end;

end.
