unit Main;

interface

uses
    Windows,
    Messages,
    SysUtils,
    Classes,
    Graphics,
    Controls,
    Forms,
    Dialogs,
    GL,
    GLUT,
    BaseTypes,
    BaseMath,
    GLClass,
    Camera,
    GLTimer,
    GLTexture,
    GLText,
    GLConsole,
    BaseMesh,
    MD2Mesh,
    Base3DObject,
    ScriptTypes,
    ScriptExceptions,
    GLBillboard,
    GLParticleSystem,
    ASEMesh,
    BoundingVolumes;

type
  TMainForm = class(TForm)
    Script: TScript;
    Funcs: TFuncs;
    QuitFunc: TFunc;
    AboutFunc: TFunc;
    ConDumpFunc: TFunc;
    GetExtensionsFunc: TFunc;
    GetRendererFunc: TFunc;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure QuitFuncEval(Sender: TObject; args: TArgList;
  var Result: Variant);
    procedure AboutFuncEval(Sender: TObject; args: TArgList;
  var Result: Variant);
    procedure ConDumpFuncEval(Sender: TObject; args: TArgList;
  var Result: Variant);
    procedure GetExtensionsFuncEval(Sender: TObject; args: TArgList;
      var Result: Variant);
    procedure GetRendererFuncEval(Sender: TObject; args: TArgList;
      var Result: Variant);
  private
    { Private declarations }
    procedure CreateComponents;
    procedure SetGLStateVariables;
    procedure ConsoleCommand(command:string);
  public
    { Public declarations }
    GL:TOpenGL;
    Camera:TFirstPersonCamera;
    RenderTimer:TGLTimer;
    fFont:TGLFont;
    fConsole:TConsole;
    msh:TMeshASE;
    sys:TParticleSystem;
    fwdVel,strafeLeft,strafeRight:TFloat;
    procedure RenderTimerTimer(Sender:TObject;LagCount:integer);
  end;

var
  MainForm: TMainForm;

implementation

uses
    Startup;

{$R *.DFM}

{ TMainForm }
procedure TMainForm.ConsoleCommand(command:string);
begin
     Script.Text:=command;
     try
        if Script.Execute then exit;
     except
           on e:EScriptError do
              fConsole.AddWrap(e.message);
           on e:Exception do
              fConsole.AddWrap('Exception : '+e.message);
     end;
end;

procedure TMainForm.CreateComponents;
begin
     RenderTimer:=TGLTimer.Create(Self);
     RenderTimer.Enabled:=false;
     RenderTimer.OnTimer:=RenderTimerTimer;
     RenderTimer.Interval:=0;
end;

procedure TMainForm.RenderTimerTimer(Sender: TObject; LagCount: integer);
var
   mPos:TPoint;
begin
     GL.ClearAll;

     Camera.SetView;
     Camera.ExtractFrustum;

     msh.Render(Camera);
     Caption:=IntToStr(msh.NumDrawn);

     sys.PreRender;
     sys.Update(GetTickCount);
     sys.Render(Camera);
     sys.PostRender;

     Setup2DScreen(ClientWidth,ClientHeight);
     fConsole.Render;

     GetCursorPos(mPos);
     SetCursorPos(Width div 2, Height div 2);

     Camera.Pitch(((ClientHeight div 2)-mPos.Y)/100*5);
     Camera.Yaw(((ClientWidth div 2)-mPos.X)/100*5);

     Camera.MoveForward(fwdVel);
     Camera.StrafeLeft(strafeLeft);
     Camera.StrafeRight(strafeRight);

     GL.Flip;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
     Randomize;
     ShowCursor(false);
     GL:=TOpenGL.Create(Self);
     with TStartupForm.Create(Self) do
     begin
          try
             FullscreenCheckBox.Checked:=true;
             FullscreenCheckBox.Enabled:=false;
             ShowModal;
             GL.XRes:=XRes;
             GL.YRes:=YRes;
             SCRWIDTH:=XRes;
             SCRHEIGHT:=YRes;
             if not FullScreen then
             begin
                  Self.ClientWidth:=XRes;
                  Self.ClientHeight:=YRes;
             end;
             GL.FullScreen:=FullScreen;
             GLClass.ColorBits:=ColorBits;
             if ModalResult=mrAbort then
                Application.Terminate
             else
                 GL.InitGL;
          finally
                 Free;
          end;
     end;

     SetGLStateVariables;

     Camera:=TFirstPersonCamera.Create;
     Camera.Pos:=MakeVector(0,2,0);
     Camera.ViewVolume.Width:=ClientWidth;
     Camera.ViewVolume.Height:=ClientHeight;
     Camera.ViewVolume.Yon:=50;

     CreateComponents;

     fFont:=TGLFont.CreateFromTGA('Data\UI\font.TGA');
     fFont.TextModePlacement:=true;
     fFont.Visible:=true;

     fConsole:=TConsole.Create(fFont);
     fConsole.Charset:=0;
     fConsole.Visible:=false;
     fConsole.Conback.LoadTexture('Data\UI\conback.JPG');
     fConsole.OnConsoleCommand:=ConsoleCommand;
     fConsole.Init;

     msh:=TMeshASE.Create;
     msh.TextureDir:='Data\Textures\';
     msh.LoadFromFile('Data\Models\pillars2.ASE');
     msh.VNormals:=true;
     msh.CalculateBoundingSphere;

     sys:=TFire.Create;
     sys.FrameInterval:=0;
     sys.InitialColor:=MakeColorf(1,0.8,0.1,0.5);
     sys.LoadParticleTexture('Data\Textures\particle.JPG');
     sys.Origin:=MakeVector(0,1,0);
     sys.ParticleSize:=0.5;
     sys.LifeDecrement:=0.1;
     sys.NumParticles:=30;

     Camera.SetView;

     RenderTimer.Enabled:=true;
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
     case Key of
          VK_ESCAPE:Close;
          VK_UP:fwdVel:=0.1;
          VK_DOWN:fwdVel:=-0.1;
          VK_LEFT:strafeLeft:=0.1;
          VK_RIGHT:strafeRight:=0.1;
     end;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
     RenderTimer.Enabled:=false;
     ShowCursor(true);
     msh.Free;

     if assigned(Camera) then Camera.Free;
     if assigned(GL) then GL.Free;
end;

procedure TMainForm.SetGLStateVariables;
begin
     glEnable(GL_DEPTH_TEST);
     glDisable(GL_LIGHTING);
     glEnable(GL_TEXTURE_2D);
     glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
     glEnable(GL_BLEND);
end;

procedure TMainForm.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
     case Key of
          VK_TAB:fConsole.Visible:=not fConsole.Visible;
          VK_UP:fwdVel:=0;
          VK_DOWN:fwdVel:=0;
          VK_LEFT:strafeLeft:=0;
          VK_RIGHT:strafeRight:=0;
     end;
end;

procedure TMainForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
     if fConsole.Visible then
        fConsole.Update(Key);
end;

procedure TMainForm.QuitFuncEval(Sender: TObject; args: TArgList;
  var Result: Variant);
begin
     Close;
end;

procedure TMainForm.AboutFuncEval(Sender: TObject; args: TArgList;
  var Result: Variant);
begin
     with fConsole do
     begin
          AddWrap('----------------------------------------------------------------------------');
          AddWrap('**Lucifer console 2.0**');
          AddWrap('Written by Ananth B. AKA HellSpawn');
          AddWrap('----------------------------------------------------------------------------');
     end;
end;

procedure TMainForm.ConDumpFuncEval(Sender: TObject; args: TArgList;
  var Result: Variant);
begin
     try
        fConsole.Lines.Add('Console dump at '+DateTimeToStr(Now));
        fConsole.Lines.SaveToFile(args[0].Eval);
     except
           on e:exception do
              fConsole.AddWrap(e.message);
     end;
end;

procedure TMainForm.GetExtensionsFuncEval(Sender: TObject; args: TArgList;
  var Result: Variant);
var
   i:integer;
begin
     for i:=0 to GetNumExtensions-1 do
         fConsole.Write(IntToStr(i+1)+'. '+GetExtension(i));
end;

procedure TMainForm.GetRendererFuncEval(Sender: TObject; args: TArgList;
  var Result: Variant);
begin
     fConsole.Write(GetRenderer);
end;

end.
